#include <cmath>

#include "camera.h"

static const int MIN_PITCH = -50.0;
static const int MAX_PITCH = +50.0;

#define deg2rad(deg) ((deg) * M_PI / 180.0)
#define rad2deg(rad) ((rad) * 180.0 / M_PI)


Camera::Camera(void) :
    m_eye(), m_lookAt(1.0, 0.0, 0.0), m_up(0.0, 1.0, 0.0)
{
    updatePitchAndYaw();
}

Camera::Camera(const Vector3 &eye, const Vector3 &view) :
    m_eye(eye), m_lookAt(eye + view.unit()), m_up(0.0, 1.0, 0.0)
{
    updatePitchAndYaw();
}

void Camera::setEye(const Vector3 &eye)
{
    Vector3 v = view();
    m_eye = eye;
    m_lookAt = m_eye + v;
}

void Camera::pitch(double degrees)
{
    m_pitch += degrees;
    if (m_pitch < MIN_PITCH)
        m_pitch = MIN_PITCH;
    if (m_pitch > MAX_PITCH)
        m_pitch = MAX_PITCH;
    updateVectors();
}

void Camera::yaw(double degrees)
{
    m_yaw += degrees;
    updateVectors();
}

void Camera::translate(const Vector3 &t)
{
    m_eye += t;
    m_lookAt += t;
}

void Camera::moveForward(double d)
{
    translate(view() * d);
}

void Camera::strafe(double d)
{
    translate(right() * d);
}

void Camera::updateVectors(void)
{
    double y = sin(deg2rad(m_pitch));
    double x = cos(deg2rad(m_pitch)) * cos(deg2rad(m_yaw));
    double z = cos(deg2rad(m_pitch)) * sin(deg2rad(m_yaw));

    m_lookAt = m_eye + Vector3(x, y, z);
}

void Camera::updatePitchAndYaw(void)
{
    double phi = asin(view().y());
    double theta = acos(view().x() / cos(phi));

    m_pitch = rad2deg(phi);
    m_yaw = rad2deg(theta);
}
