#ifndef _MESH_H_
#define _MESH_H_

#include "vector3.h"
#include "movable.h"

#include <istream>
#include <vector>
using std::istream;

#include <cfloat>
#include <GL/glut.h>


class Triangle
{
    public:
    int v1, v2, v3;     // vertex indices
    Vector3 normal;
    Vector3 color;

    Triangle(int v1, int v2, int v3) :
        v1(v1), v2(v2), v3(v3), normal(0.0, 1.0, 0.0) {}
    Triangle(void) :
        v1(-1), v2(-1), v3(-1), normal(0.0, 1.0, 0.0) {}
};

class Mesh
{
    public:
    std::vector<Vector3> vertices;
    std::vector<Vector3> normals;
    std::vector<Triangle> faces;

    Vector3 center;
    double min_x, max_x;
    double min_y, max_y;
    double min_z, max_z;
    double max_extent;

    Mesh(void) :
        center(0.0, 0.0, 0.0),
        min_x(FLT_MAX), max_x(FLT_MIN),
        min_y(FLT_MAX), max_y(FLT_MIN),
        min_z(FLT_MAX), max_z(FLT_MIN),
        max_extent(1.0)
    {}

    void addVertex(Vector3 v)
    {
        vertices.push_back(v);
        normals.push_back(Vector3(0, 0, 0));
    }

    void addTriangle(Triangle t)
    {
        faces.push_back(t);
    }

    void draw(void) const;
    void drawNormals(void) const;

    static Mesh *read(istream &is);

    private:
    void drawTriangle(const Triangle &t) const;
    void drawNormal(const Triangle &t) const;
};

#endif
