#ifndef _VIEWPORT_H_
#define _VIEWPORT_H_

#include "vector.h"
#include "vector3.h"

class Viewport
{
    private:
        int m_width;
        int m_height;
        float m_top;
        float m_bottom;
        float m_near;
        float m_far;
        float m_left;
        float m_right;

    public:
        Viewport(int width, int height) :
            m_width(width), m_height(height),
            m_top(2.0f), m_bottom(-2.0f), m_near(0.01f), m_far(30.0f),
            m_left(-2.0f * (float)width/height),
            m_right(2.0f * (float)width/height)
        {}

        void set(int width, int height);

        float ratio(void) const { return (float)m_width/m_height; }
        int width(void) const { return m_width; }
        int height(void) const { return m_height; }
        float top(void) const { return m_top; }
        float bottom(void) const { return m_bottom; }
        float left(void) const { return m_left; }
        float right(void) const { return m_right; }
        float near(void) const { return m_near; }
        float far(void) const { return m_far; }

        Vector3 trackball(int x, int y) const;

        /* pixel to world coordinate transforms */
        Vector p2i(int x, int y) const;
        Vector i2w(Vector w) const;
        Vector p2w(int x, int y) const { return i2w(p2i(x, y)); }
};

#endif
